import 'package:flutter/material.dart';
import 'package:millui/millui.dart';
import 'dart:math';

import '../app_provider.dart';

class Refash extends StatefulWidget {
  _Refash createState() => new _Refash();
}

class _Refash extends State<Refash> {
  List _items = [
    1,
    2,
    3,
    4,
    5,
    6,
    7,
    8,
  ];
  bool _visibileSel = false;
  _refresh() async {
    await Future.delayed(Duration(milliseconds: 1000), () {
      if (mounted) {
        setState(() {
          _items = [
            1,
            2,
            3,
          ];
        });
      }
    });
  }

  _onLoad() async {
    await Future.delayed(Duration(milliseconds: 200), () {
      if (mounted) {
        _items.add(1);
        _items.add(2);
        _items.add(3);
        _items.add(4);
        _items.add(5);
        setState(() {
          _items = _items;
        });
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    bool isCn = getLanage(context);

    return Scaffold(
      appBar: AppBar(
        title: Text('refresh'),
      ),
      body: Stack(
        children: [
          Stack(
            children: [
              Column(
                children: [
                  Container(
                    height: 30,
                    child: Text('Top'),
                  ),
                  Expanded(
                    flex: 1,
                    child: MillRefresh(
                      timeout: 6,
                      sliverList: _buildList(_items),
                      dropText: isCn ? '下拉刷新' : 'pulldown ↓',
                      releaseText: isCn ? '释放加载' : 'release ↑',
                      footReleaseText: isCn ? '释放加载' : 'release ↓',
                      noDataText: isCn ? '没有更多数据' : 'NO DATA',
                      refreshing: MillBarBeatLoading(),
                      loading: MillBallRotateLoading(size: 2, paintSize: 16,),
                      onRefresh: _refresh,
                      onLoad: _onLoad,
                      preAdapter: [
                        SliverToBoxAdapter(
                          child: Container(
                            alignment: Alignment.center,
                            height: 60,
                            child: Text(isCn ? '你好' : 'hellow'),
                          ),
                        ),
                        SliverPersistentHeader(
                          pinned: true, //是否固定在顶部
                          floating: true,
                          delegate: _SliverAppBarDelegate(
                            minHeight: 40, //收起的高度
                            maxHeight: 40, //展开的最大高度
                            child: Container(
                              alignment: Alignment.center,
                              child: Text(isCn ? '吸顶导航' : 'fix'),
                              color: Colors.red,
                            ),
                          ),
                        ),
                      ],
                    ),
                  ),
                ],
              ),
              Positioned(
                  left: 0,
                  top: 60,
                  child: Offstage(
                      offstage: !_visibileSel,
                      child: GestureDetector(
                          onTap: () {
                            setState(() {
                              _visibileSel = false;
                            });
                          },
                          child: Opacity(
                            opacity: 0.6,
                            child: Container(
                              alignment: Alignment(0, 0),
                              width: double.maxFinite,
                              height: double.maxFinite,
                              decoration:
                                  BoxDecoration(color: Color(0xff000000)),
                              child: Text('s'),
                            ),
                          )))),
            ],
          )
        ],
      ),
    );
  }
}

Widget _buildList(items) {
  return SliverList(
      delegate: SliverChildBuilderDelegate(
    (context, index) {
      return Container(
        padding: EdgeInsets.all(20),
        child: Text('${items[index]}'),
      );
    },
    childCount: items.length,
  ));
}

class _SliverAppBarDelegate extends SliverPersistentHeaderDelegate {
  _SliverAppBarDelegate({
    @required this.minHeight,
    @required this.maxHeight,
    @required this.child,
  });

  final double minHeight;
  final double maxHeight;
  final Widget child;

  @override
  double get minExtent => minHeight;

  @override
  double get maxExtent => max(maxHeight, minHeight);

  @override
  Widget build(
      BuildContext context, double shrinkOffset, bool overlapsContent) {
    return new SizedBox.expand(child: child);
  }

  @override
  bool shouldRebuild(_SliverAppBarDelegate oldDelegate) {
    return maxHeight != oldDelegate.maxHeight ||
        minHeight != oldDelegate.minHeight ||
        child != oldDelegate.child;
  }
}
