import 'package:flutter/material.dart';
import './field.dart';
import './checkbox.dart';

class MillActionSheet {
  static open(
    context, {
    GlobalKey<ScaffoldState> scaffoldState,
    bool showCancel = false,
    bool showConfirm = false,
    String title,
    double radius = 15,
    double height,
    String labelAlias = 'name',
    String valueAlias = 'value',
    List list = const [],
    Widget child,
    String listType = 'value',
    Function onSelect,
    Function onConfirm,
    Color backgroundColor = const Color(0xffffffff),
    Color confirmBgColor = const Color(0xff1989fa),
    List defaultChecked,
    TextStyle textStyle = const TextStyle(
      color: Color(0xff323233),
      fontSize: 16,
    ),
    TextStyle cancelTextStyle = const TextStyle(
      color: Color(0xff646566),
      fontSize: 16,
    ),
    TextStyle confirmTextStyle = const TextStyle(
      color: Color(0xffffffff),
      fontSize: 16,
    ),
    TextStyle titleTextStyle = const TextStyle(
      color: Color(0xff969799),
      fontSize: 16,
    ),
    String cancelText = '取消',
    String confirmText = '确定',
    Function onCancel,
  }) {
    Radius r = Radius.circular(radius);
    Widget box = LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      double maxheight = constraints.maxHeight - 50;
      if (showCancel || showConfirm) {
        maxheight -= 58;
      }

      if (title != null) {
        maxheight -= 61;
      }

      return ActionSheetBox(
        showCancel: showCancel,
        showConfirm: showConfirm,
        onConfirm: onConfirm,
        height: height ?? maxheight,
        label: labelAlias,
        value: valueAlias,
        list: list,
        child: child,
        onSelect: onSelect,
        textStyle: textStyle,
        cancelTextStyle: cancelTextStyle,
        confirmTextStyle: confirmTextStyle,
        confirmBgColor: confirmBgColor,
        backgroundColor: backgroundColor,
        title: title,
        titleTextStyle: titleTextStyle,
        listType: listType,
        defaultChecked: defaultChecked,
        cancelText: cancelText,
        confirmText: confirmText,
        onCancel: onCancel,
      );
    });
    if (scaffoldState == null) {
      return showModalBottomSheet(
        isScrollControlled: true,
        context: context,
        backgroundColor: backgroundColor,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.only(topLeft: r, topRight: r),
        ),
        clipBehavior: Clip.antiAlias,
        builder: (BuildContext context) {
          return box;
        },
      );
    } else {
      return showBottomSheet(
        context: context,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.only(topLeft: r, topRight: r),
        ),
        backgroundColor: backgroundColor,
        builder: (BuildContext context) {
          return box;
        },
      );
    }
  }
}

class ActionSheetBox extends StatefulWidget {
  ActionSheetBox({
    Key key,
    this.showCancel,
    this.showConfirm,
    this.confirmBgColor,
    this.height,
    this.list,
    this.onSelect,
    this.label,
    this.value,
    this.cancelTextStyle,
    this.confirmTextStyle,
    this.textStyle,
    this.backgroundColor,
    this.child,
    this.title,
    this.titleTextStyle,
    this.listType,
    this.onConfirm,
    this.defaultChecked,
    this.cancelText,
    this.confirmText,
    this.onCancel,
  }) : super(key: key);
  final bool showCancel;
  final bool showConfirm;
  final Color confirmBgColor;
  final double height;
  final List list;
  final Function onSelect;
  final String label;
  final String value;
  final TextStyle cancelTextStyle;
  final TextStyle confirmTextStyle;
  final TextStyle textStyle;
  final Color backgroundColor;
  final Widget child;
  final String title;
  final TextStyle titleTextStyle;
  final String listType;
  final Function onConfirm;
  final List defaultChecked;
  final String cancelText;
  final String confirmText;
  final Function onCancel;

  _ActionSheetBox createState() => new _ActionSheetBox();
}

class _ActionSheetBox extends State<ActionSheetBox> {
  List defaultValue = [];

  @override
  void initState() {
    defaultValue = widget.defaultChecked;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    List<Widget> children = [];
    if (widget.title != null) {
      children.add(Padding(
        padding: EdgeInsets.only(left: 10, right: 10, bottom: 5),
        child: Container(
          decoration: BoxDecoration(
            border: Border(
              bottom: BorderSide(
                color: Color(0xffebedf0),
                width: 1,
              ),
            ),
          ),
          height: 60,
          alignment: Alignment.center,
          child: Text(
            widget.title,
            style: widget.titleTextStyle,
          ),
        ),
      ));
    }

    Widget box = widget.child == null
        ? Container(
            height: widget.height,
            child: ListView(
              children: widget.list.map((val) {
                if (widget.listType == 'value') {
                  return buildValue(val, context);
                } else if (widget.listType == 'kv') {
                  return buildKv(val);
                } else {
                  return buildCheckbox(val);
                }
              }).toList(),
            ),
          )
        : Container(
            padding: EdgeInsets.only(top: 15),
            child: widget.child,
          );
    children.add(box);

    Widget cancel = FlatButton(
      color: Color(0xffffffff),
      highlightColor: Color(0xfff2f3fa),
      child: Container(
        height: 50,
        alignment: Alignment.center,
        child: Text(
          widget.cancelText ?? '取消',
          style: widget.cancelTextStyle,
        ),
      ),
      onPressed: () {
        if (widget.onCancel != null) {
          widget.onCancel();
        }
        Navigator.pop(context);
      },
    );

    if (widget.showConfirm) {
      children.addAll([
        Container(
          height: 8,
          color: Color(0xfff2f3fa),
        ),
        Row(
          children: [
            Expanded(
              flex: 1,
              child: cancel,
            ),
            Container(
                color: Color(0xffcccccc), padding: EdgeInsets.only(left: 2)),
            Expanded(
              flex: 1,
              child: Container(
                color: widget.confirmBgColor,
                child: FlatButton(
                  color: widget.confirmBgColor,
                  child: Container(
                    height: 50,
                    alignment: Alignment.center,
                    child: Text(
                      widget.confirmText ?? '确定',
                      style: widget.confirmTextStyle,
                    ),
                  ),
                  onPressed: () {
                    if (widget.onConfirm != null) {
                      Map res = {'context': context};
                      if (widget.listType == 'checkbox') {
                        res['result'] = defaultValue;
                      }
                      widget.onConfirm(res);
                    }
                    Navigator.pop(context);
                  },
                ),
              ),
            )
          ],
        ),
      ]);
    }
    if (widget.showCancel) {
      children.addAll([
        Container(
          height: 8,
          color: Color(0xfff2f3fa),
        ),
        cancel,
      ]);
    }
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: children,
    );
  }

  Widget buildKv(val) {
    return MillField(
      padding: EdgeInsets.only(left: 20, right: 20),
      label: val[widget.label],
      readonly: true,
      labelWidth: 120,
      labelColor: Color(0xff666666),
      readBox: Text(
        '${val[widget.value]}',
        maxLines: 1,
        overflow: TextOverflow.ellipsis,
      ),
    );
  }

  Widget buildValue(val, context) {
    TextStyle style = widget.textStyle;
    if (val['color'] != null && val['disabled'] != true) {
      style = widget.textStyle.copyWith(color: val['color']);
    } else if (val['disabled'] == true) {
      style = widget.textStyle.copyWith(color: Color(0xffbcbec2));
    }
    return FlatButton(
      textColor: val['color'],
      color: val['backgroundColor'] ?? widget.backgroundColor,
      highlightColor: val['backgroundColor'] == null ? Color(0xfff2f3fa) : null,
      disabledColor: Color(0xffe8e9ee),
      child: Container(
        height: 50,
        alignment: Alignment.center,
        child: Text(
          val[widget.label],
          style: style,
        ),
      ),
      onPressed: val['disabled'] != true
          ? () {
              if (widget.onSelect != null) {
                widget.onSelect(val);
                Navigator.pop(context);
              }
            }
          : null,
    );
  }

  Widget buildCheckbox(val) {
    TextStyle style = widget.textStyle;
    if (val['color'] != null && val['disabled'] != true) {
      style = widget.textStyle.copyWith(color: val['color']);
    } else if (val['disabled'] == true) {
      style = widget.textStyle.copyWith(color: Color(0xffbcbec2));
    }

    return Container(
      color: val['backgroundColor'],
      padding: EdgeInsets.only(left: 20, right: 20),
      child: MillCheckBox(
        value: defaultValue.indexOf(val[widget.value]) >= 0,
        onChange: () {
          _change(val);
        },
        outside: 12,
        disabled: val['disabled'] == true,
        size: 20,
        label: val[widget.label],
        labelStyle: style,
      ),
    );
  }

  _change(val) {
    int index = defaultValue.indexOf(val[widget.value]);
    if (index >= 0) {
      defaultValue.removeAt(index);
    } else {
      defaultValue.add(val[widget.value]);
    }
    setState(() {
      defaultValue = defaultValue;
    });
  }
}
