import 'package:flutter/material.dart';

class MillDialog {
  static tip(
    context, {
    key,
    bool barrierDismissible = false,
    Color titleColor = const Color(0xff323233),
    Color backgroundColor = const Color(0xffffffff),
    Color messageColor = const Color(0xff646566),
    Color buttonTextColor = const Color(0xff333333),
    String title,
    String message,
    String buttonText = '确定',
    Function onConfirm,
    double width = 286,
    double radius = 12,
  }) {
    showDialog(
      context: context,
      barrierDismissible: barrierDismissible,
      builder: (BuildContext context) {
        return Tip(
          key: key,
          message: message,
          title: title,
          buttonText: buttonText,
          onConfirm: () {
            if (onConfirm != null) {
              onConfirm();
            }
            Navigator.pop(context);
          },
          width: width,
          radius: radius,
          backgroundColor: backgroundColor,
          messageColor: messageColor,
          buttonTextColor: buttonTextColor,
          titleColor: titleColor,
        );
      },
    );
  }

  static confirm(
    context, {
    key,
    bool barrierDismissible = false,
    Color titleColor = const Color(0xff323233),
    Color backgroundColor = const Color(0xffffffff),
    Color messageColor = const Color(0xff646566),
    Color confirmTextColor = const Color(0xff409eff),
    Color cancelTextColor = const Color(0xff333333),
    String title,
    String message,
    String confirmText = '确定',
    String cancelText = '取消',
    Function onConfirm,
    Function onCancel,
    double width = 286,
    double radius = 12,
  }) {
    showDialog(
      context: context,
      barrierDismissible: barrierDismissible,
      builder: (BuildContext context) {
        return new Confirm(
          key: key,
          message: message,
          title: title,
          confirmText: confirmText,
          cancelText: cancelText,
          onConfirm: () {
            if (onConfirm != null) {
              onConfirm();
            }
            Navigator.pop(context);
          },
          onCancel: () {
            if (onCancel != null) {
              onCancel();
            }
            Navigator.pop(context);
          },
          width: width,
          radius: radius,
          backgroundColor: backgroundColor,
          messageColor: messageColor,
          confirmTextColor: confirmTextColor,
          cancelTextColor: cancelTextColor,
          titleColor: titleColor,
        );
      },
    );
  }
}

class Tip extends StatelessWidget {
  Tip({
    Key key,
    this.backgroundColor,
    this.title,
    this.message,
    this.buttonText,
    this.onConfirm,
    this.width,
    this.messageColor,
    this.buttonTextColor,
    this.titleColor,
    this.radius,
  }) : super(key: key);
  final Color backgroundColor;
  final Color messageColor;
  final Color buttonTextColor;
  final Color titleColor;
  final String title;
  final String message;
  final String buttonText;
  final Function onConfirm;
  final double width;
  final double radius;

  @override
  Widget build(BuildContext context) {
    return new Material(
        type: MaterialType.transparency, //透明类型
        child: Center(
          child: Container(
            width: width,
            padding: EdgeInsets.only(top: 10),
            decoration: BoxDecoration(
              color: backgroundColor,
              borderRadius: BorderRadius.circular(radius),
            ),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              crossAxisAlignment: CrossAxisAlignment.stretch,
              children: <Widget>[
                Visibility(
                  visible: title != null,
                  child: Container(
                    alignment: Alignment.center,
                    padding: EdgeInsets.fromLTRB(20, 15, 20, 0),
                    child: Text(
                      title ?? '提示',
                      textAlign: TextAlign.center,
                      style: TextStyle(
                        color: titleColor,
                        fontSize: 16,
                        fontWeight: FontWeight.w500,
                      ),
                    ),
                  ),
                ),
                Container(
                  child: Container(
                    alignment: Alignment.center,
                    padding: EdgeInsets.fromLTRB(24, 18, 24, 24),
                    child: Text(
                      message,
                      textAlign: TextAlign.center,
                      style: TextStyle(
                        color: messageColor,
                        fontSize: 15,
                      ),
                    ),
                  ),
                ),
                Container(
                  height: 50,
                  decoration: BoxDecoration(
                    border: Border(
                      top: BorderSide(
                        width: 0.8,
                        color: Color(0xffebedf4),
                      ),
                    ),
                  ),
                  child: FlatButton(
                    color: backgroundColor,
                    highlightColor: Color(0xffe6e6e6),
                    textColor: buttonTextColor,
                    shape: RoundedRectangleBorder(
                        borderRadius: BorderRadius.only(
                      bottomLeft: Radius.circular(radius),
                      bottomRight: Radius.circular(radius),
                    )),
                    onPressed: onConfirm,
                    child: Text(buttonText),
                  ),
                ),
              ],
            ),
          ),
        ));
  }
}

class Confirm extends StatelessWidget {
  Confirm({
    Key key,
    this.backgroundColor,
    this.title,
    this.message,
    this.confirmText,
    this.cancelText,
    this.onConfirm,
    this.onCancel,
    this.width,
    this.messageColor,
    this.confirmTextColor,
    this.cancelTextColor,
    this.titleColor,
    this.radius,
  }) : super(key: key);
  final Color backgroundColor;
  final Color messageColor;
  final Color confirmTextColor;
  final Color cancelTextColor;
  final Color titleColor;
  final String title;
  final String message;
  final String confirmText;
  final String cancelText;
  final Function onConfirm;
  final Function onCancel;
  final double width;
  final double radius;
  @override
  Widget build(BuildContext context) {
    return new Material(
      type: MaterialType.transparency, //透明类型
      child: Center(
        child: Container(
          width: width,
          padding: EdgeInsets.only(top: 10),
          decoration: BoxDecoration(
            color: backgroundColor,
            borderRadius: BorderRadius.circular(radius),
          ),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            crossAxisAlignment: CrossAxisAlignment.stretch,
            children: <Widget>[
              Visibility(
                visible: title != null,
                child: Container(
                  alignment: Alignment.center,
                  padding: EdgeInsets.fromLTRB(20, 15, 20, 0),
                  child: Text(
                    title ?? '提示',
                    textAlign: TextAlign.center,
                    style: TextStyle(
                      color: titleColor,
                      fontSize: 16,
                      fontWeight: FontWeight.w500,
                    ),
                  ),
                ),
              ),
              Container(
                child: Container(
                  alignment: Alignment.center,
                  padding: EdgeInsets.fromLTRB(24, 18, 24, 24),
                  child: Text(
                    message,
                    textAlign: TextAlign.center,
                    style: TextStyle(
                      color: messageColor,
                      fontSize: 15,
                    ),
                  ),
                ),
              ),
              Container(
                height: 50,
                decoration: BoxDecoration(
                  border: Border(
                    top: BorderSide(
                      width: 0.8,
                      color: Color(0xffebedf4),
                    ),
                  ),
                ),
                child: Row(
                  crossAxisAlignment: CrossAxisAlignment.stretch,
                  children: <Widget>[
                    Expanded(
                      flex: 1,
                      child: FlatButton(
                        color: backgroundColor,
                        highlightColor: Color(0xffe6e6e6),
                        textColor: cancelTextColor,
                        shape: RoundedRectangleBorder(
                          borderRadius: BorderRadius.only(
                            bottomLeft: Radius.circular(radius),
                          ),
                        ),
                        onPressed: onCancel,
                        child: Text(cancelText),
                      ),
                    ),
                    Container(
                      decoration: BoxDecoration(
                        border: Border(
                          right: BorderSide(
                            width: 1,
                            color: Color(0xffe6e6e6),
                          ),
                        ),
                      ),
                    ),
                    Expanded(
                        flex: 1,
                        child: FlatButton(
                          color: backgroundColor,
                          highlightColor: Color(0xffe6e6e6),
                          textColor: confirmTextColor,
                          shape: RoundedRectangleBorder(
                            borderRadius: BorderRadius.only(
                              bottomRight: Radius.circular(radius),
                            ),
                          ),
                          onPressed: onConfirm,
                          child: Text(confirmText),
                        )),
                  ],
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
