import 'package:flutter/material.dart';

class MillDropDown extends StatefulWidget {
  MillDropDown({
    this.onChange,
    this.children = const [],
    this.activeColor = const Color(0xff1989fa),
    this.bgColor = const Color(0xffffffff),
    this.activeBgColor = const Color(0xffffffff),
    this.direction = 'down', // down or up
  });
  final List<MillDropDownItem> children;
  final Color activeColor;
  final Color bgColor;
  final Color activeBgColor;
  final Function onChange;
  final String direction;

  _MillDropDown createState() => _MillDropDown();
}

class _MillDropDown extends State<MillDropDown> {
  OverlayEntry _newEntry;
  GlobalKey _pkey = GlobalKey();
  Key _key;
  Map<Key, dynamic> _states;
  List<Option> _keys;

  @override
  void initState() {
    _states = {};
    super.initState();
  }

  @override
  dispose() {
    _newEntry?.remove();
    _newEntry = null;
    super.dispose();
  }

  _closer() {
    _newEntry?.remove();
    _newEntry = null;
    setState(() {
      _key = null;
    });
  }

  _select(data) {
    if (_states == null) {
      _states = {};
    }
    _states[_key] = data;
    if (widget.onChange != null) {
      widget.onChange(_key, data);
    }
    setState(() {
      _states = _states;
      _key = null;
    });
    _newEntry?.remove();
    _newEntry = null;
  }

  _change(Option opt) {
    Key key = opt?.key;
    Widget child = opt?.child;
    List options = opt?.options;

    _newEntry?.remove();
    _newEntry = null;
    RenderBox box = _pkey.currentContext.findRenderObject();
    Offset offset = box.localToGlobal(Offset.zero);
    Size size = MediaQuery.of(context).size;
    double mheight = size.height;
    double mwidth = size.width;
    String direction = widget.direction;
    if ((mheight - offset.dy <= 190 + 60) && widget.direction == 'down') {
      direction = 'up';
    }

    if (widget.direction == 'up' && offset.dy <= 251) {
      direction = 'down';
    }

    catDetails(TapUpDetails details) {
      int length = widget.children.length;
      double dur = mwidth / length;
      if (details.globalPosition.dy > offset.dy &&
          details.globalPosition.dy <= offset.dy + 60) {
        int index = (details.globalPosition.dx / dur).floor();
        if (index >= 0 && index < length && _keys[index] is Option) {
          if (_keys[index].callBack != null) {
            _keys[index].callBack();
            _change(Option());
          } else if (_keys[index].key != null && !_keys[index].disabled) {
            _change(_keys[index]);
          }
        } else {
          _change(Option());
        }
      }
    }

    if (key != null && _key != key) {
      _newEntry = OverlayEntry(builder: (context) {
        return Positioned(
          left: 0,
          right: 0,
          // top: direction == 'down' ? offset.dy + 60 : 0,
          // bottom: direction == 'down' ? 0 : mheight - offset.dy,
          top: 0,
          bottom: 0,
          child: Material(
            color: Colors.transparent,
            child: Container(
              color: Colors.transparent,
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.stretch,
                children: [
                  direction == 'down'
                      ? GestureDetector(
                          onTapUp: catDetails,
                          child: Container(
                            color: Colors.transparent,
                            height: offset.dy + 60,
                          ),
                        )
                      : Expanded(
                          child: GestureDetector(
                            onTap: () {
                              _closer();
                            },
                            child: Container(
                              color: Color(0x72000000),
                            ),
                          ),
                          flex: 1,
                        ),
                  Slide(
                    child: Container(
                      constraints: BoxConstraints(
                        maxHeight: 190,
                      ),
                      color: Color(0xffffffff),
                      child: child != null
                          ? child
                          : ListView(
                              padding: EdgeInsets.all(0),
                              children: options
                                  .map(
                                    (el) => FlatButton(
                                      onPressed: el['disabled'] == true
                                          ? () {}
                                          : () {
                                              _select(el);
                                            },
                                      child: Row(
                                        mainAxisAlignment:
                                            MainAxisAlignment.spaceBetween,
                                        children: [
                                          Text(
                                            el['name'],
                                            style: TextStyle(
                                              color: _states[_key] != null &&
                                                      _states[_key]['id'] ==
                                                          el['id']
                                                  ? widget.activeColor
                                                  : el['disabled'] == true
                                                      ? Color(0xff999999)
                                                      : Color(0xff333333),
                                            ),
                                          ),
                                          _states[_key] != null &&
                                                  _states[_key]['id'] ==
                                                      el['id']
                                              ? Icon(
                                                  Icons.check,
                                                  size: 16,
                                                  color: widget.activeColor,
                                                )
                                              : Container()
                                        ],
                                      ),
                                    ),
                                  )
                                  .toList(),
                            ),
                    ),
                  ),
                  direction == 'down'
                      ? Expanded(
                          child: GestureDetector(
                            onTap: () {
                              _closer();
                            },
                            child: Container(
                              color: Color(0x72000000),
                            ),
                          ),
                          flex: 1,
                        )
                      : GestureDetector(
                          onTapUp: catDetails,
                          child: Container(
                            height: mheight - offset.dy,
                            color: Colors.transparent,
                          ),
                        ),
                ],
              ),
            ),
          ),
        );
      });

      if (_newEntry != null) {
        Overlay.of(context).insert(_newEntry);
      }
      setState(() {
        _key = key;
      });
    } else {
      setState(() {
        _key = null;
      });
    }
  }

  String _getTitleByState(Key k, List options, defaultValue) {
    if (_states == null || _states[k] == null) {
      if (defaultValue != null) {
        var data = options.firstWhere((el) => el['id'] == defaultValue);
        if (data != null) {
          _states[k] = data;
          setState(() {
            _states = _states;
          });
        }
        return data != null ? data['name'] : '';
      } else if (options.length > 0) {
        if (options[0] != null) {
          _states[k] = options[0];
        }
        setState(() {
          _states = _states;
        });
        return options[0]['name'];
      }
      return '';
    } else {
      return _states[k]['name'] ?? '';
    }
  }

  @override
  Widget build(BuildContext context) {
    _keys = [];
    return Container(
      key: _pkey,
      color: _newEntry != null ? widget.activeBgColor : widget.bgColor,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceAround,
        children: widget.children.map((el) {
          if (el.builder != null && el.builder is MillDropDownBuilder) {
            MillDropDownCustomItem obj = el.builder(context);
            _keys.add(Option(callBack: obj.onTap));
            return Expanded(
              flex: 1,
              child: GestureDetector(
                onTap: obj.onTap,
                child: obj.child,
              ),
            );
          } else {
            Key k = ObjectKey(el);
            Key elKey = el.key ?? k;
            _keys.add(Option(
              key: elKey,
              child: el.child,
              options: el.options,
              disabled: el.disabled == true,
            ));
            return MillDropDownItemBox(
              key: elKey,
              options: el.options,
              child: el.child,
              title: el.title ??
                  _getTitleByState(elKey, el.options, el.defalutValue),
              change: _change,
              isExpanded: _key != null && _key == elKey,
              activeColor: widget.activeColor,
              disabled: el.disabled == true,
            );
          }
        }).toList(),
      ),
    );
  }
}

typedef MillDropDownBuilder = MillDropDownCustomItem Function(
    BuildContext context);

class MillDropDownItem {
  MillDropDownItem({
    this.key,
    this.child,
    this.options,
    this.defalutValue,
    this.builder,
    this.title,
    this.disabled,
  });

  final Key key;
  final Widget child;
  final List options;
  final defalutValue;
  final String title;
  final MillDropDownBuilder builder;
  final bool disabled;
}

class MillDropDownCustomItem {
  MillDropDownCustomItem({
    @required this.child,
    this.onTap,
  });

  final Widget child;
  final Function onTap;
}

class MillDropDownItemBox extends StatefulWidget {
  MillDropDownItemBox({
    this.key,
    this.child,
    this.options,
    this.change,
    this.title,
    this.isExpanded,
    this.activeColor,
    this.disabled,
  }) : super(key: key);
  final Key key;
  final Widget child;
  final List options;
  final Function change;
  final String title;
  final bool isExpanded;
  final Color activeColor;
  final bool disabled;

  _MillDropDownItemBox createState() => _MillDropDownItemBox();
}

class _MillDropDownItemBox extends State<MillDropDownItemBox> {
  @override
  Widget build(BuildContext context) {
    return Expanded(
      flex: 1,
      child: GestureDetector(
        onTap: () {
          if (!widget.disabled) {
            widget.change(Option(
              key: widget.key,
              child: widget.child,
              options: widget.options,
            ));
          }
        },
        child: Container(
          height: 60,
          alignment: Alignment.center,
          color: Colors.transparent,
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            mainAxisSize: MainAxisSize.max,
            children: [
              Text(
                widget.title ?? 'title',
                style: TextStyle(
                  color: widget.isExpanded
                      ? widget.activeColor
                      : widget.disabled
                          ? Color(0xff999999)
                          : Color(0xff333333),
                ),
              ),
              widget.isExpanded
                  ? Icon(
                      Icons.arrow_drop_up_outlined,
                      color: widget.activeColor,
                    )
                  : Icon(
                      Icons.arrow_drop_down_outlined,
                      color: widget.disabled
                          ? Color(0xffc3c3c3)
                          : Color(0xff999999),
                    ),
            ],
          ),
        ),
      ),
    );
  }
}

class Slide extends StatefulWidget {
  Slide({
    Key key,
    this.child,
  }) : super(key: key);

  final Widget child;

  @override
  _Slide createState() => _Slide();
}

class _Slide extends State<Slide> with SingleTickerProviderStateMixin {
  showIt() {
    _animationController.forward();
  }

  hideIt() {
    _animationController.reverse();
  }

  AnimationController _animationController;
  Animation _fadeAnimation;

  @override
  void initState() {
    _animationController = AnimationController(
      vsync: this,
      duration: const Duration(milliseconds: 200),
    );
    _fadeAnimation =
        CurvedAnimation(parent: _animationController, curve: Curves.easeIn);
    super.initState();

    showIt();
  }

  @override
  void deactivate() {
    _animationController.stop();
    super.deactivate();
  }

  @override
  void dispose() {
    _animationController?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SizeTransition(
      axis: Axis.vertical,
      sizeFactor: new Tween(begin: 0.2, end: 1.0).animate(_fadeAnimation),
      child: SizedBox(
        width: double.infinity,
        child: Material(
          color: Colors.transparent,
          child: widget.child,
        ),
      ),
    );
  }
}

class Option {
  Option({
    this.key,
    this.child,
    this.options,
    this.disabled,
    this.callBack,
  });
  final Key key;
  final child;
  final List options;
  final bool disabled;
  final Function callBack;
}
