import 'package:flutter/animation.dart';
import 'package:flutter/material.dart';
import './loading_common.dart';

class MillBallScaleLoading extends StatefulWidget {
  const MillBallScaleLoading({
    Key key,
    this.size = 10.0,
    this.color = Colors.blue,
    this.duration = const Duration(milliseconds: 800),
    this.curve = Curves.linear,
    this.padding = const EdgeInsets.symmetric(horizontal: 3),
  }) : super(key: key);

  final EdgeInsets padding;
  final double size;
  final Color color;
  final Duration duration;
  final Curve curve;

  @override
  _MillBallScaleLoadingState createState() => _MillBallScaleLoadingState();
}

class _MillBallScaleLoadingState extends State<MillBallScaleLoading>
    with SingleTickerProviderStateMixin {
  AnimationController _controller;
  Animation _animation;
  @override
  void initState() {
    _controller = AnimationController(vsync: this, duration: widget.duration)
      ..repeat();

    _animation = _controller.drive(CurveTween(curve: widget.curve));

    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: List.generate(3, (index) {
        return Padding(
          padding: widget.padding,
          child: ScaleTransition(
            scale: DelayTween(begin: 0.0, end: 1.0, delay: index * .2)
                .animate(_animation),
            child: SizedBox(
              width: widget.size,
              height: widget.size,
              child: DecoratedBox(
                decoration: BoxDecoration(
                  shape: BoxShape.circle,
                  color: widget.color,
                ),
              ),
            ),
          ),
        );
      }),
    );
  }
}

class MillBallWaveLoading extends StatefulWidget {
  final Duration duration;
  final Curve curve;
  final double size;
  final Color color;
  final EdgeInsets padding;

  const MillBallWaveLoading({
    Key key,
    this.size = 10.0,
    this.padding = const EdgeInsets.symmetric(horizontal: 3),
    this.color = Colors.blue,
    this.duration = const Duration(milliseconds: 800),
    this.curve = Curves.linear,
  }) : super(key: key);
  @override
  _MillBallWaveLoadingState createState() => _MillBallWaveLoadingState();
}

class _MillBallWaveLoadingState extends State<MillBallWaveLoading>
    with SingleTickerProviderStateMixin {
  AnimationController _controller;
  List<Animation> _animations = [];

  @override
  void initState() {
    _controller =
        AnimationController(vsync: this, duration: widget.duration)
          ..repeat();
    List.generate(3, (index) {
      _animations.add(DelayTween(begin: 0.0, end: 0.8, delay: 0.2 * index)
          .animate(CurvedAnimation(parent: _controller, curve: widget.curve)));
    });
    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Row(
        mainAxisAlignment: MainAxisAlignment.center,
      children: List.generate(3, (index) {
        return Padding(
          padding: widget.padding,
          child: AnimatedBuilder(
            animation: _controller,
            builder: (context, child) {
              return Align(
                alignment: Alignment(0.0, 0.4 * _animations[index].value),
                child: SizedBox(
                  width: widget.size,
                  height: widget.size,
                  child: DecoratedBox(
                    decoration: BoxDecoration(
                      shape: BoxShape.circle,
                      color: widget.color,
                    ),
                  ),
                ),
              );
            },
          ),
        );
      }),
    );
  }
}

class MillBallRotateLoading extends StatefulWidget {
  final double size;
  final double paintSize; 
  final Color color;
  final Duration duration;
  final Curve curve;
  final int count;

  const MillBallRotateLoading({
    Key key,
    this.size = 4.0,
    this.paintSize = 28,
    this.color = Colors.blue,
    this.count = 8,
    this.duration = const Duration(milliseconds: 1200),
    this.curve = Curves.linear,
  }) : super(key: key);

  @override
  _MillBallRotateLoadingState createState() =>
      _MillBallRotateLoadingState();
}

class _MillBallRotateLoadingState extends State<MillBallRotateLoading>
    with SingleTickerProviderStateMixin {
  AnimationController _controller;
  Animation _animation;
  List<double> opacity = [0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.9, 1.0];
  List<List<BallStyle>> _styles = [];

  @override
  void initState() {
    _controller = AnimationController(vsync: this, duration: widget.duration)
      ..repeat();

    _animation = _controller.drive(CurveTween(curve: widget.curve));

    List.generate(opacity.length, (index) {
      List<double> _opacityIndexList = [];
      _opacityIndexList
          .addAll(opacity.sublist(opacity.length - index, opacity.length));
      _opacityIndexList.addAll(opacity.sublist(0, opacity.length - index));

      List<BallStyle> _styleList = [];
      for (int i = 0; i < opacity.length; i++) {
        _styleList.add(
          BallStyle(
            size: widget.size ?? 2.0,
            color: widget.color?.withOpacity(_opacityIndexList[i]),
          ),
        );
      }
      _styles.add(_styleList);
    });

    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: _controller,
      builder: (context, child) {
        var index = (_animation.value * opacity.length).floor();
        return CustomPaint(
          size: Size(widget.paintSize, widget.paintSize),
          painter: BallPainter(
            ballStyles: _styles[index],
            count: widget.count,
          ),
        );
      },
    );
  }
}

class MillBarBeatLoading extends StatefulWidget {
  final double width;
  final double height;
  final Color color;
  final BorderRadiusGeometry borderRadius;
  final Duration duration;
  final Curve curve;
  final EdgeInsets padding;

  const MillBarBeatLoading({Key key,
    this.width = 2.0,
    this.height = 12.0,
    this.color = Colors.blue,
    this.padding = const EdgeInsets.symmetric(horizontal: 3),
    this.borderRadius = const BorderRadius.only(
      topLeft: Radius.circular(3),
      topRight: Radius.circular(3),
    ),
    this.duration = const Duration(milliseconds: 800),
    this.curve = Curves.linear,
  }) : super(key: key);

  @override
  _MillBarBeatLoadingState createState() => _MillBarBeatLoadingState();
}

class _MillBarBeatLoadingState extends State<MillBarBeatLoading>
    with SingleTickerProviderStateMixin {
  AnimationController _controller;

  @override
  void initState() {
    _controller = AnimationController(vsync: this, duration: widget.duration)
      ..repeat();
    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: List.generate(5, (index) {
        return ScaleTransition(
          scale: DelayTween(begin: 1.0, end: 2.3, delay: index * .2).animate(
              CurvedAnimation(parent: _controller, curve: widget.curve)),
          child: Padding(
            padding: widget.padding,
            child: SizedBox(
              width: widget.width,
              height: widget.height,
              child: DecoratedBox(
                decoration: BoxDecoration(
                  shape: BoxShape.rectangle,
                  color: widget.color,
                  borderRadius: widget.borderRadius,
                ),
              ),
            ),
          ),
        );
      }),
    );
  }
}
