import 'package:flutter/material.dart';
import 'package:flutter/animation.dart';
import 'dart:math' as math;

class DelayTween extends Tween<double> {
  final double delay;

  DelayTween({double begin, double end, this.delay})
      :super(begin: begin, end: end);

  @override
  double lerp(double t) {
    return super.lerp((math.sin((t - delay) * 2 * math.pi) + 1) / 2);
  }

  @override
  double evaluate(Animation<double> animation) => lerp(animation.value);
}

class BallPainter extends CustomPainter {
  final List<BallStyle> ballStyles;
  final int count;

  Paint _paint = Paint();

  BallPainter({
    this.ballStyles = const [
      BallStyle(
        size: 3,
        color: Colors.blue,
      ),
    ],
    this.count = 2,
  });

  @override
  void paint(Canvas canvas, Size size) {
    double _radius = math.min(size.width, size.height) / 2;
    double perAngle = 2 * math.pi / count;
    List.generate(count, (index) {
      int styleIndex = math.min(ballStyles.length-1, index);
      BallStyle _style = ballStyles[styleIndex];
      _paint
          ..color = _style.color
          ..style = PaintingStyle.fill;

      canvas.drawCircle(
          Offset(_radius + _radius * math.cos(perAngle * index),
              _radius + _radius * math.sin(perAngle * index)),
          _style.size,
          _paint);
    });
  }

  @override
  bool shouldRepaint(covariant BallPainter old) {
    return count != old.count || ballStyles != old.ballStyles;
  }
}

class BallStyle {
  final double size;
  final Color color;
  const BallStyle({
    this.size,
    this.color
  });

  BallStyle copyWith({
    double size,
    Color color,
  }) {
    return BallStyle(
      size: size ?? this.size,
      color: color ?? this.color,
    );
  }
}
