import 'package:flutter/material.dart';
import 'dart:async';
import './icon.dart';

Map _status = {
  'success': {
    'icon': MillIcons.success_circle_fill,
    'label': '成功',
    'color': Color(0xff67c23a),
    'size': 22.0,
  },
  'error': {
    'icon': MillIcons.close_circle_fill,
    'label': '错误',
    'color': Color(0xfff56c6c),
    'size': 25.0,
  },
  'warning': {
    'icon': MillIcons.warning_circle_fill,
    'label': '警告',
    'color': Color(0xffe6a23c),
    'size': 25.0,
  },
  'message': {
    'icon': MillIcons.message_file,
    'label': '消息',
    'color': Color(0xff909399),
    'size': 25.0,
  },
  'default': {
    'icon': null,
    'label': '提示',
    'color': Color(0xff909399),
    'size': 25.0,
  }
};

class MillNotice {
  static String success = 'success';
  static String error = 'error';
  static String warning = 'warning';
  static String message = 'message';
  static String defaultType = 'default';

  BuildContext Context;
  double top;
  double bottom;

  MillNotice({
    this.Context,
    this.bottom = 20.0,
    this.top = 20.0,
  });

  Widget _getDefaultWidget(child, [double width, String type, String til]) {
    Map status = _status[type];
    IconData icon = status['icon'];
    String title = til ?? status['label'];
    Color color = status['color'];
    double size = status['size'];

    return Container(
      width: width,
      height: 78,
      alignment: Alignment.centerLeft,
      padding: EdgeInsets.symmetric(horizontal: 15, vertical: 10),
      decoration: BoxDecoration(
        border: Border.all(color: Color(0xfff2f2f2)),
        borderRadius: BorderRadius.circular(6),
        color: Color(0xffffffff),
      ),
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          type == 'default'
              ? Container()
              : Container(
                  alignment: Alignment.topLeft,
                  padding: EdgeInsets.only(top: 0, right: 18),
                  child: Icon(
                    icon,
                    color: color,
                    size: size,
                  ),
                ),
          Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Text(
                title,
                style: TextStyle(
                  color: Color(0xff444444),
                  fontSize: 16,
                  fontWeight: FontWeight.bold,
                ),
              ),
              Padding(
                padding: EdgeInsets.only(top: 8),
              ),
              Text(
                child,
                style: TextStyle(
                  color: Color(0xff666666),
                  fontSize: 14,
                ),
              ),
            ],
          ),
        ],
      ),
    );
  }

  leftTop({
    child,
    int duration = 3,
    BuildContext context,
    double height,
    int maxCount = 3,
    String type = 'default',
    double top,
    String title,
  }) {
    custom(
      child: child,
      duration: duration,
      context: context,
      tween: new Tween(
        begin: Offset(-2, 0.0),
        end: Offset(0.0, 0.0),
      ),
      left: 10.0,
      top: top ?? this.top,
      height: height,
      maxCount: maxCount,
      type: type,
      title: title,
    );
  }

  rightTop({
    child,
    int duration = 3,
    BuildContext context,
    double height,
    int maxCount = 3,
    String type = 'default',
    double top,
    String title,
  }) {
    custom(
      child: child,
      duration: duration,
      context: context,
      tween: new Tween(
        begin: Offset(2, 0.0),
        end: Offset(0.0, 0.0),
      ),
      right: 10.0,
      top: top ?? this.top,
      height: height,
      maxCount: maxCount,
      type: type,
      title: title,
    );
  }

  centerTop({
    child,
    int duration = 3,
    BuildContext context,
    double height,
    int maxCount = 3,
    String type = 'default',
    double top,
    String title,
  }) {
    double _height = height;
    dynamic nchild = child;
    if (child is String) {
      _height = 78.0;
      BuildContext c = context ?? Context;
      double width = MediaQuery != null ? MediaQuery.of(c).size.width : 350;
      nchild = Padding(
        padding: EdgeInsets.symmetric(horizontal: 15),
        child: _getDefaultWidget(child, width - 30, type, title),
      );
    }
    custom(
      child: nchild,
      duration: duration,
      context: context,
      tween: new Tween(
        begin: Offset(0.0, -5.5),
        end: Offset(0.0, 0.0),
      ),
      left: 0.0,
      top: top ?? this.top,
      height: _height,
      maxCount: maxCount,
    );
  }

  centerBottom({
    child,
    int duration = 3,
    BuildContext context,
    double height,
    int maxCount = 3,
    String type = 'default',
    double bottom,
    String title,
  }) {
    double _height = height;
    dynamic nchild = child;
    if (child is String) {
      _height = 78.0;
      BuildContext c = context ?? Context;
      double width = MediaQuery != null ? MediaQuery.of(c).size.width : 350;
      nchild = _getDefaultWidget(child, width - 30, type, title);
    }
    custom(
      child: Padding(
        padding: EdgeInsets.symmetric(horizontal: 15),
        child: nchild,
      ),
      duration: duration,
      context: context,
      tween: new Tween(
        begin: Offset(0.0, 5.5),
        end: Offset(0.0, 0.0),
      ),
      left: 0.0,
      bottom: bottom ?? this.bottom,
      height: _height,
      maxCount: maxCount,
    );
  }

  custom({
    child,
    int duration,
    BuildContext context,
    tween,
    double left,
    double right,
    double top,
    double bottom,
    double height,
    int maxCount,
    String type,
    String title,
  }) {
    if (tween == null) {
      tween = new Tween(
        begin: Offset(2, 0.0),
        end: Offset(0.0, 0.0),
      );
    }
    if (context == null) {
      context = Context;
    }
    _MillNotice notice = _MillNotice();
    notice.init(context);
    double _height = 78.0;
    dynamic newchild = child;
    if (child is String) {
      double width =
          MediaQuery != null ? MediaQuery.of(context).size.width : 289;
      newchild = _getDefaultWidget(child, width * 0.7, type, title);
    } else {
      _height = height;
    }
    notice.notice(
      child: newchild,
      duration: duration,
      tween: tween,
      left: left,
      right: right,
      top: top,
      bottom: bottom,
      height: _height,
      maxCount: maxCount,
    );
  }
}

class _MillNotice {
  BuildContext context;
  static final _MillNotice _instance = _MillNotice._internal();

  factory _MillNotice() {
    return _instance;
  }

  init(BuildContext context) {
    _instance.context = context;
  }

  _MillNotice._internal();
  List<LocalKey> _overlayQueue = [];
  Listener listener = new Listener();

  notice({
    Widget child,
    int duration,
    tween,
    double left,
    double right,
    double top,
    double bottom,
    double height,
    int maxCount,
  }) {
    GlobalKey<NoticeStateFulState> noticeStateKey = GlobalKey();
    Widget newChild = _NoticeStateFul(
      key: noticeStateKey,
      child: child,
      duration: Duration(seconds: duration),
      tween: tween,
    );
    if (_overlayQueue.length < maxCount) {
      LocalKey key = UniqueKey();
      Timer _timer = null;
      _overlayQueue.add(key);
      OverlayEntry _newEntry;
      _newEntry = OverlayEntry(builder: (context) {
        return NoticeItem(
            key: key,
            keys: _overlayQueue,
            top: top,
            bottom: bottom,
            left: left,
            right: right,
            listener: listener,
            child: newChild,
            height: height,
            remove: () {
              Timer(Duration(milliseconds: 100), () {
                if (_timer != null && _timer.isActive) {
                  _timer?.cancel();
                  _timer = null;

                  noticeStateKey?.currentState?.hideIt();
                  Future.delayed(Duration(milliseconds: 300), () {
                    int index = _overlayQueue.indexWhere((k) => k == key);
                    if (index > -1) {
                      _overlayQueue.removeAt(index);
                    }
                    if (_newEntry != null) {
                      _newEntry?.remove();
                      _newEntry = null;
                    }
                  });
                }
              });
            });
      });
      if (_newEntry != null) {
        Overlay.of(context).insert(_newEntry);

        if (_overlayQueue.length > 0) {
          listener.emit(_overlayQueue);
        }
        _timer = Timer(Duration(seconds: duration), () {
          Future.delayed(Duration(milliseconds: 360), () {
            _timer?.cancel();
            _timer = null;
            if (_newEntry != null) {
              int index = _overlayQueue.indexWhere((k) => k == key);
              if (index > -1) {
                _overlayQueue.removeAt(index);
              }
              _newEntry?.remove();
              _newEntry = null;
            }
          });
        });
      }
    }
  }
}

class NoticeItem extends StatefulWidget {
  NoticeItem({
    this.key,
    this.keys,
    this.child,
    this.top,
    this.left,
    this.right,
    this.bottom,
    this.listener,
    this.height,
    this.remove,
  }) : super(key: key);

  final LocalKey key;
  final Widget child;
  final List keys;
  final Listener listener;
  final double top;
  final double left;
  final double right;
  final double bottom;
  final double height;
  final Function remove;
  _NoticeItem createState() => _NoticeItem();
}

class _NoticeItem extends State<NoticeItem> {
  List keys = [];

  @override
  void initState() {
    keys = widget.keys;
    widget.listener.addEventListener((list) {
      setState(() {
        keys = list;
      });
    });
    super.initState();
  }

  @override
  void dispose() {
    widget.listener.dispose();
    super.dispose();
  }

  double getPosi(double posi, int index) {
    if (posi != null && index > -1) {
      return posi + index * (widget.height + 6);
    } else {
      return posi;
    }
  }

  @override
  Widget build(BuildContext context) {
    int index = keys.indexWhere((k) => k == widget.key);
    bool _removing = false;

    _remove() {
      if (!_removing) {
        _removing = true;
        widget.remove();
        Timer(Duration(milliseconds: 400), () {
          _removing = false;
        });
      }
    }

    return Positioned(
      top: getPosi(widget.top, index),
      bottom: getPosi(widget.bottom, index),
      left: widget.left,
      right: widget.right,
      child: GestureDetector(
        onHorizontalDragUpdate: (DragUpdateDetails details) {
          if ((widget.left != null &&
                  widget.right == null &&
                  details.delta.dx < -3) ||
              (widget.left == null &&
                  widget.right != null &&
                  details.delta.dx > 3)) {
            _remove();
          }
        },
        onVerticalDragUpdate: (DragUpdateDetails details) {
          if ((widget.bottom != null &&
                  widget.top == null &&
                  details.delta.dy > 3) ||
              (widget.bottom == null &&
                  widget.top != null &&
                  details.delta.dy < -3)) {
            _remove();
          }
        },
        child: widget.child,
      ),
    );
  }
}

class Listener {
  static final Listener _instance = Listener._internal();

  factory Listener() {
    return _instance;
  }

  dispose() {
    queue = [];
  }

  Listener._internal();

  List queue = [];

  addEventListener(Function fn) {
    this.queue.add(fn);
  }

  emit(list) {
    for (int i = 0, l = queue.length; i < l; i++) {
      this.queue[i](list);
    }
  }
}

class _NoticeStateFul extends StatefulWidget {
  _NoticeStateFul({
    Key key,
    this.child,
    this.duration,
    this.tween,
  }) : super(key: key);

  final Widget child;
  final Duration duration;
  final Animatable tween;

  @override
  NoticeStateFulState createState() => NoticeStateFulState();
}

class NoticeStateFulState extends State<_NoticeStateFul>
    with SingleTickerProviderStateMixin {
  showIt() {
    _animationController.forward();
  }

  hideIt() {
    _animationController.reverse();
    _timer?.cancel();
  }

  AnimationController _animationController;
  Animation _fadeAnimation;

  Timer _timer;

  @override
  void initState() {
    _animationController = AnimationController(
      vsync: this,
      duration: const Duration(milliseconds: 350),
    );
    _fadeAnimation =
        CurvedAnimation(parent: _animationController, curve: Curves.easeIn);
    super.initState();

    showIt();
    _timer = Timer(widget.duration, () {
      hideIt();
    });
  }

  @override
  void deactivate() {
    _timer?.cancel();
    _animationController.stop();
    super.deactivate();
  }

  @override
  void dispose() {
    _timer?.cancel();
    _animationController?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SlideTransition(
      position: widget.tween.animate(_fadeAnimation),
      child: Center(
        child: Material(
          color: Colors.transparent,
          child: widget.child,
        ),
      ),
    );
  }
}
