import 'package:flutter/material.dart';
import './scroll_notification_listener.dart';
import './scroll_physics.dart';
import './slive_header.dart';
import './slive_footer.dart';

class MillRefresh extends StatefulWidget {
  MillRefresh({
    Key key,
    @required this.sliverList,
    this.onRefresh,
    this.onLoad,
    this.refreshFinished,
    this.refreshing,
    this.loading,
    this.headerBgColor = const Color(0xfff3f3f3),
    this.footerBgColor = const Color(0xfff3f3f3),
    this.preAdapter,
    this.sufAdapter,
    this.timeout = 10,
    this.error,
    this.hasMore = true,
    this.dropText = '下拉刷新',
    this.releaseText = '释放加载',
    this.footReleaseText = '释放加载',
    this.noDataText = '没有更多数据',
  }) : super(key: key);

  final sliverList;
  final Widget refreshFinished;
  final Widget refreshing;
  final Widget loading;
  final Color headerBgColor;
  final Color footerBgColor;
  final Function onRefresh;
  final Function onLoad;
  final Widget error;
  final timeout;
  final bool hasMore;
  final List<Widget> preAdapter;
  final SliverToBoxAdapter sufAdapter;
  final String dropText;
  final String releaseText;
  final String footReleaseText;
  final String noDataText;

  @override
  _MillRefresh createState() => new _MillRefresh();
}

class _MillRefresh extends State<MillRefresh> {
  ValueNotifier<BouncingSettings> _bouncingNotifier;
  MillRefreshPhysics _physics;
  ValueNotifier<bool> _focusNotifier;
  ValueNotifier<double> _extraExtentNotifier;

  @override
  void initState() {
    super.initState();
    _extraExtentNotifier = ValueNotifier<double>(0.0);
    _focusNotifier = ValueNotifier<bool>(false);
    _bouncingNotifier = ValueNotifier<BouncingSettings>(BouncingSettings());
    _createPhysics();
  }

  void _createPhysics() {
    _bouncingNotifier.value = BouncingSettings(
      top: true,
      bottom: true,
    );
    _physics = MillRefreshPhysics(
      bouncingNotifier: _bouncingNotifier,
      extraExtentNotifier: _extraExtentNotifier,
    );
  }

  // 销毁
  void dispose() {
    _focusNotifier.dispose();
    _extraExtentNotifier.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    List<Widget> sliverList = [];

    if (widget.sliverList is SliverList ||
        widget.sliverList is SliverToBoxAdapter) {
      sliverList.add(widget.sliverList);
    } else if (widget.sliverList is List) {
      sliverList.addAll(widget.sliverList);
    }

    if (widget.preAdapter != null) {
      sliverList.insertAll(0, widget.preAdapter);
    }

    if (widget.sufAdapter != null) {
      sliverList.add(widget.sufAdapter);
    }

    if (widget.onRefresh != null) {
      sliverList.insert(
        0,
        new SliveHeader(
          timeout: widget.timeout,
          error: widget.error,
          finished: widget.refreshFinished,
          loading: widget.refreshing,
          onRefresh: widget.onRefresh,
          focusNotifier: _focusNotifier,
          backgroundColor: widget.headerBgColor,
          dropText: widget.dropText,
          releaseText: widget.releaseText,
        ),
      );
    }
    if (widget.onLoad != null) {
      sliverList.add(
        new SliveFooter(
          hasMore: widget.hasMore,
          loading: widget.loading,
          onLoad: widget.onLoad,
          focusNotifier: _focusNotifier,
          backgroundColor: widget.footerBgColor,
          extraExtentNotifier: _extraExtentNotifier,
          footReleaseText: widget.footReleaseText,
          noDataText: widget.noDataText,
        ),
      );
    }

    return ScrollNotificationListener(
      onFocus: (focus) {
        _focusNotifier.value = focus;
      },
      child: ScrollConfiguration(
        behavior: ScrollConfiguration.of(context),
        child: CustomScrollView(
          physics: _physics,
          slivers: sliverList,
        ),
      ),
    );
  }
}
