import 'package:flutter/material.dart';

class MillSearch extends StatefulWidget {
  MillSearch({
    Key key,
    this.backgroundColor = const Color(0xffffffff),
    this.fillColor = const Color(0xfff7f8fa),
    this.textColor = const Color(0xff4f4f4f),
    this.showButton = false,
    this.disabled = false,
    this.inputRadio = 2,
    this.padding = const EdgeInsets.all(10),
    this.hintText = '请输入',
    this.onCancel,
    this.onChanged,
    this.onSubmitted,
    this.value,
    this.radius = 3,
    this.maxHeight = 60,
    this.buttonText = '取消',
    this.buttonTextColor,
    this.showIcon = true,
    this.prefixIcon,
    this.contentPadding = const EdgeInsets.all(15),
  }) : super(key: key);
  final Color backgroundColor;
  final Color fillColor;
  final Color textColor;
  final bool showButton;
  final bool disabled;
  final double inputRadio;
  final EdgeInsets padding;
  final String hintText;
  final Function onCancel;
  final Function onChanged;
  final Function onSubmitted;
  final double radius;
  final double maxHeight;
  final String value;
  final String buttonText;
  final Color buttonTextColor;
  final bool showIcon;
  final Widget prefixIcon;
  final EdgeInsets contentPadding;

  _Search createState() => _Search();
}

class _Search extends State<MillSearch> {
  String _query;
  bool _focus = false;
  TextEditingController _queryController = TextEditingController();
  FocusNode _focusNode = FocusNode();

  @override
  void initState() {
    super.initState();
    _query = widget.value ?? '';
    _queryController.text = widget.value;
    _focusNode.addListener(() {
      setState(() {
        _focus = _focusNode.hasFocus;
      });
    });
  }

  @override
  void didUpdateWidget(oldWidget) {
    if (oldWidget.value != widget.value) {
      _queryController.text = widget.value;
      _query = widget.value ?? '';
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    _queryController.dispose();
    _focusNode.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: widget.backgroundColor,
      padding: widget.padding,
      child: Row(
        children: [
          Expanded(
            flex: 1,
            child: ClipRRect(
              borderRadius: BorderRadius.circular(widget.radius),
              child: ConstrainedBox(
                constraints: BoxConstraints(maxHeight: widget.maxHeight),
                child: TextField(
                  focusNode: _focusNode,
                  controller: _queryController,
                  onChanged: (query) {
                    setState(() {
                      _query = query;
                    });
                    if (widget.onChanged != null) {
                      widget.onChanged(query);
                    }
                  },
                  onSubmitted: (query) {
                    if (widget.onSubmitted != null) {
                      widget.onSubmitted(query);
                    }
                  },
                  cursorRadius: Radius.circular(20),
                  maxLines: 1,
                  textAlignVertical: TextAlignVertical.center,
                  style: TextStyle(
                    fontSize: 16,
                    color: widget.textColor,
                  ),
                  decoration: InputDecoration(
                    contentPadding: widget.contentPadding,
                    hintText: widget.hintText,
                    hintStyle:
                        TextStyle(color: Color(0xffcccccc), fontSize: 14),
                    fillColor: widget.fillColor,
                    filled: !widget.disabled,
                    border: InputBorder.none,
                    prefixIcon: widget.showIcon
                        ? widget.prefixIcon ??
                            Icon(
                              Icons.search,
                              color: widget.textColor,
                              size: 20,
                            )
                        : null,
                    suffixIcon: _query != '' && _query is String && _focus
                        ? GestureDetector(
                            onTap: () {
                              setState(() {
                                _query = '';
                              });
                              _queryController.clear();
                            },
                            child: Icon(
                              Icons.close,
                              color: widget.textColor,
                              size: 20,
                            ),
                          )
                        : null,
                  ),
                ),
              ),
            ),
          ),
          widget.showButton
              ? FlatButton(
                  padding: EdgeInsets.symmetric(vertical: 5, horizontal: 5),
                  minWidth: 0,
                  onPressed: () {
                    if (widget.onCancel != null) {
                      widget.onCancel();
                    }
                  },
                  child: Text(
                    widget.buttonText ?? '取消',
                    style: TextStyle(
                        color: widget.buttonTextColor ?? Color(0xff666666)),
                  ),
                )
              : Container(),
        ],
      ),
    );
  }
}
