import 'package:flutter/material.dart';
import 'dart:async';

class MillToast {
  static BuildContext fcontext = null;
  static setContext(BuildContext context) {
    fcontext = context;
  }

  static loading(
    context, {
    String text = '加载中',
    int duration = 2,
    Color backgroundColor = const Color(0xff444444),
    Color textColor = const Color(0xffffffff),
    Color iconBgColor = const Color(0xff444444),
    Animation<Color> valueColor =
        const AlwaysStoppedAnimation(Color(0xffffffff)),
  }) {
    var toast = _MillToast();
    if (context == null && fcontext != null) {
      context = fcontext;
    }
    toast.init(context);
    Widget child = new Container(
      constraints: BoxConstraints(
        minWidth: 138,
        maxWidth: 140,
        minHeight: 138,
        maxHeight: 280,
      ),
      child: new Container(
        decoration: ShapeDecoration(
          color: backgroundColor,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.all(
              Radius.circular(8.0),
            ),
          ),
        ),
        child: new Column(
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            new Padding(
                padding: const EdgeInsets.only(
                  top: 15.0,
                ),
                child: new CircularProgressIndicator(
                  backgroundColor: iconBgColor,
                  valueColor: valueColor,
                )),
            new Container(
              padding: const EdgeInsets.only(
                top: 22.0,
                left: 15.0,
                right: 15.0,
                bottom: 15.0,
              ),
              child: new Text(
                text,
                textAlign: TextAlign.center,
                maxLines: 12,
                style: new TextStyle(fontSize: 14.0, color: textColor),
              ),
            ),
          ],
        ),
      ),
    );
    toast.toast(
      child: child,
      duration: duration,
    );
  }

  static success(
    context, {
    String text = '成功',
    int duration = 2,
    Color backgroundColor = const Color(0xff444444),
    Color textColor = const Color(0xffffffff),
  }) {
    MillToast.toast(
      context,
      text: text,
      duration: duration,
      backgroundColor: backgroundColor,
      textColor: textColor,
    );
  }

  static warning(
    context, {
    String text = '警告',
    int duration = 2,
    Color backgroundColor = const Color(0xff444444),
    Color textColor = const Color(0xffffffff),
  }) {
    MillToast.toast(context,
        text: text,
        duration: duration,
        backgroundColor: backgroundColor,
        textColor: textColor,
        icon: 0xe550);
  }

  static error(
    context, {
    String text = '错误',
    int duration = 2,
    Color backgroundColor = const Color(0xff444444),
    Color textColor = const Color(0xffffffff),
  }) {
    MillToast.toast(context,
        text: text,
        duration: duration,
        backgroundColor: backgroundColor,
        textColor: textColor,
        icon: 0xe662);
  }

  static text(context,
      {String text = '成功',
      int duration = 2,
      Color backgroundColor = const Color(0xff444444),
      Color textColor = const Color(0xffffffff)}) {
    var toast = _MillToast();
    if (context == null && fcontext != null) {
      context = fcontext;
    }
    toast.init(context);
    Widget child = new Container(
        padding: EdgeInsets.fromLTRB(25, 20, 25, 20),
        decoration: ShapeDecoration(
          color: backgroundColor,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.all(
              Radius.circular(8.0),
            ),
          ),
        ),
        child: Text(
          text,
          textAlign: TextAlign.center,
          style: new TextStyle(fontSize: 14.0, color: textColor),
        ));
    toast.toast(
      child: child,
      duration: duration,
    );
  }

  static toast(
    context, {
    String text = '成功',
    int duration = 2,
    Color backgroundColor = const Color(0xff444444),
    Color textColor = const Color(0xffffffff),
    int icon = 0xe56a,
  }) async {
    var toast = _MillToast();

    if (context == null && fcontext != null) {
      context = fcontext;
    }
    toast.init(context);
    Widget child = new Container(
      constraints: BoxConstraints(
        minWidth: 138,
        maxWidth: 140,
        minHeight: 138,
        maxHeight: 280,
      ),
      child: new Container(
        decoration: ShapeDecoration(
          color: backgroundColor,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.all(
              Radius.circular(8.0),
            ),
          ),
        ),
        child: new Column(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: <Widget>[
            new Padding(
                padding: const EdgeInsets.only(
                  top: 15.0,
                ),
                child: Icon(IconData(icon, fontFamily: 'MilluiIcons'),
                    size: 44, color: textColor)),
            new Container(
              padding: const EdgeInsets.only(
                top: 15.0,
                left: 15.0,
                right: 15.0,
                bottom: 10.0,
              ),
              child: new Text(
                text,
                textAlign: TextAlign.center,
                maxLines: 12,
                style: new TextStyle(
                  fontSize: 14.0,
                  color: textColor,
                ),
              ),
            ),
          ],
        ),
      ),
    );
    toast.toast(
      child: child,
      duration: duration,
    );
  }
}

class _MillToast {
  BuildContext context;
  static final _MillToast _instance = _MillToast._internal();

  factory _MillToast() {
    return _instance;
  }

  init(BuildContext context) {
    _instance.context = context;
  }

  _MillToast._internal();

  OverlayEntry _entry;
  List<_ToastEntry> _overlayQueue = List();
  Timer _timer;

  _showOverlay() {
    if (_overlayQueue.length == 0) {
      _entry = null;
      return;
    }
    _ToastEntry _toastEntry = _overlayQueue.removeAt(0);
    _entry = _toastEntry.entry;
    if (context == null)
      throw ("Error: Context is null, Please call init(context) before showing toast.");
    Overlay.of(context).insert(_entry);

    _timer = Timer(_toastEntry.duration, () {
      Future.delayed(Duration(milliseconds: 360), () {
        removeCustomToast();
      });
    });
  }

  removeCustomToast() {
    _timer?.cancel();
    _timer = null;
    if (_entry != null) _entry.remove();
    _showOverlay();
  }

  toast({Widget child, int duration}) {
    Widget newChild = _ToastStateFul(
      child,
      Duration(seconds: duration),
    );
    OverlayEntry newEntry = OverlayEntry(builder: (context) {
      return Positioned(
          top: 50.0, bottom: 50.0, left: 24.0, right: 24.0, child: newChild);
    });

    if (_overlayQueue.length < 3) {
      _overlayQueue.add(
          _ToastEntry(entry: newEntry, duration: Duration(seconds: duration)));
    }
    if (_timer == null) _showOverlay();
  }
}

class _ToastEntry {
  final OverlayEntry entry;
  final Duration duration;

  _ToastEntry({this.entry, this.duration});
}

class _ToastStateFul extends StatefulWidget {
  _ToastStateFul(this.child, this.duration, {Key key}) : super(key: key);

  final Widget child;
  final Duration duration;

  @override
  ToastStateFulState createState() => ToastStateFulState();
}

class ToastStateFulState extends State<_ToastStateFul>
    with SingleTickerProviderStateMixin {
  showIt() {
    _animationController.forward();
  }

  hideIt() {
    _animationController.reverse();
    _timer?.cancel();
  }

  AnimationController _animationController;
  Animation _fadeAnimation;

  Timer _timer;

  @override
  void initState() {
    _animationController = AnimationController(
      vsync: this,
      duration: const Duration(milliseconds: 350),
    );
    _fadeAnimation =
        CurvedAnimation(parent: _animationController, curve: Curves.easeIn);
    super.initState();

    showIt();
    _timer = Timer(widget.duration, () {
      hideIt();
    });
  }

  @override
  void deactivate() {
    _timer?.cancel();
    _animationController.stop();
    super.deactivate();
  }

  @override
  void dispose() {
    _timer?.cancel();
    _animationController?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return FadeTransition(
      opacity: _fadeAnimation,
      child: Center(
        child: Material(
          color: Colors.transparent,
          child: widget.child,
        ),
      ),
    );
  }
}
