import 'package:flutter/material.dart';
import './checkbox.dart';

class TreeNode extends StatefulWidget {
  TreeNode({
    this.data,
    this.onChange,
    this.alasLabel = 'label',
    this.alasValue = 'value',
    this.showCheckeds = 1,
  });
  final String alasLabel;
  final String alasValue;
  final Map data;
  final Function onChange;
  final int showCheckeds;

  @override
  _RegionNode createState() => new _RegionNode();
}

class _RegionNode extends State<TreeNode> {
  bool _expand = false;
  @override
  void initState() {
    _expand = widget.data['stack'] < 1 ||
        widget.data['stack'] < 2 && widget.data['checked'] != 0;
    super.initState();
  }

  _setTreeState(data, state) {
    Map<String, dynamic> result = {
      'checked': state,
      'stack': data['stack'],
      '${widget.alasLabel}': data[widget.alasLabel],
      '${widget.alasValue}': data[widget.alasValue],
      'disabled': data['disabled'],
    };
    if (data['children'] != null && data['children'].length > 0) {
      result['children'] =
          data['children'].map((el) => _setTreeState(el, state)).toList();
    } else {
      result['children'] = [];
    }
    return result;
  }

  _changeState() {
    int state = widget.data['checked'] != 1 ? 1 : 0;
    if (widget.onChange != null) {
      var tree = _setTreeState(widget.data, state);
      widget.onChange(tree);
    }
  }

  _toggle() {
    setState(() {
      _expand = !_expand;
    });
  }

  _onChildChange(val) {
    if (widget.onChange != null) {
      Map<String, dynamic> result = {
        'checked': 0,
        'stack': widget.data['stack'],
        '${widget.alasLabel}': widget.data[widget.alasLabel],
        '${widget.alasValue}': widget.data[widget.alasValue],
        'disabled': widget.data['disabled'],
      };
      var index = widget.data['children']
          .indexWhere((el) => el[widget.alasValue] == val[widget.alasValue]);
      result['children'] = [...widget.data['children']];
      result['children'][index] = val;
      var selected = result['children'].where((val) => val['checked'] == 1);
      var im = result['children'].where((val) => val['checked'] == 2);

      if (selected.length > 0) {
        result['checked'] =
            selected.length == widget.data['children'].length ? 1 : 2;
      } else if (im.length > 0) {
        result['checked'] = 2;
      }
      widget.onChange(result);
    }
  }

  int _getChildrenCheckedNum(list) {
    return list.where((el) => el['checked'] == 1).length;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: Color(0xffffffff),
      child: Column(
        children: <Widget>[
          Visibility(
            visible: widget.data[widget.alasLabel] != null &&
                widget.data[widget.alasLabel] != "",
            child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: <Widget>[
                  MillCheckBox(
                    disabled: widget.data['disabled'],
                    size: 22,
                    outside: 8,
                    value: widget.data['checked'] == 1,
                    indeterminate: widget.data['checked'] == 2,
                    onChange: _changeState,
                  ),
                  Padding(
                    padding: EdgeInsets.only(left: 10),
                  ),
                  Expanded(
                      flex: 1,
                      child: GestureDetector(
                          behavior: HitTestBehavior.translucent,
                          onTap: () {
                            if (!widget.data['disabled']) {
                              _changeState();
                            }
                          },
                          child: Container(
                            color: Color(0xffffffff),
                            padding: EdgeInsets.only(bottom: 8, top: 8),
                            child: Text(
                              widget.data[widget.alasLabel],
                              maxLines: 1,
                              style: TextStyle(fontSize: 16),
                            ),
                          ))),
                  Offstage(
                      offstage: widget.data['children'].length == 0 ||
                          widget.data['stack'] < 1,
                      child: GestureDetector(
                          behavior: HitTestBehavior.translucent,
                          onTap: _toggle,
                          child: Container(
                            width: 130,
                            color: Color(0xffffffff),
                            alignment: Alignment.centerRight,
                            padding: EdgeInsets.only(bottom: 8, top: 8),
                            child: widget.data['stack'] >= widget.showCheckeds
                                ? Container(
                                    width: 16,
                                    height: 16,
                                    alignment: Alignment.center,
                                    decoration: BoxDecoration(
                                      color: Color(0xffececec),
                                      borderRadius: BorderRadius.circular(4),
                                    ),
                                    child: Text(
                                      '${_getChildrenCheckedNum(widget.data['children'])}',
                                      style: TextStyle(fontSize: 14),
                                    ),
                                  )
                                : Container(),
                          ))),
                  Offstage(
                    offstage: !(widget.data['children'] != null &&
                        widget.data['children'].length > 0),
                    child: GestureDetector(
                        behavior: HitTestBehavior.translucent,
                        onTap: _toggle,
                        child: Container(
                          padding: EdgeInsets.fromLTRB(10, 10, 0, 10),
                          child: !_expand
                              ? Icon(
                                  Icons.keyboard_arrow_down,
                                  size: 25,
                                )
                              : Icon(
                                  Icons.keyboard_arrow_up,
                                  size: 25,
                                ),
                        )),
                  )
                ]),
          ),
          Visibility(
            visible: widget.data['children'] != null &&
                widget.data['children'].length > 0 &&
                _expand,
            child: Padding(
              padding: EdgeInsets.only(
                  left: widget.data[widget.alasLabel] != null &&
                          widget.data[widget.alasLabel] != ""
                      ? 20
                      : 0),
              child: Column(
                children: widget.data['children']
                    .map<Widget>((item) => TreeNode(
                          data: item,
                          onChange: _onChildChange,
                          alasLabel: widget.alasLabel,
                          alasValue: widget.alasValue,
                          showCheckeds: widget.showCheckeds,
                        ))
                    .toList(),
              ),
            ),
          )
        ],
      ),
    );
  }
}
